indexing
	description: "Objects that finds relations between persons in the genealogical database"
	author: "Thomas Gubler and Wolfgang Schwaiger"
	date: "$Date: 2008-05-20 21:01:05 +0200 (Tue, 20 May 2008) $"
	revision: "$Revision: 14 $"

class
	RELATION_FINDER

create
	make

feature -- Initialization

	make (person_list: LIST[PERSON1]; start_node, end_node : PERSON1) is
			-- Initializes the Relation finder with the person list
			do
				m_persons := person_list
				m_start_node := start_node
				m_end_node := end_node

				m_marriage_weight := 1
				m_adoption_weight := 1
				m_natural_parent_weight := 1
			end

feature{NONE} -- Internal Members

	m_persons : LIST[PERSON1]
	m_start_node : PERSON1
	m_end_node : PERSON1

	m_marriage_weight : INTEGER
	m_adoption_weight : INTEGER
	m_natural_parent_weight : INTEGER

feature -- Configuration

	set_marriage_weight (a_weight: INTEGER) is
			-- Sets the marriage weight. Set -1 to disallow marriages relations
			do
				m_marriage_weight := a_weight
			end

	set_adoption_weight (a_weight: INTEGER) is
		-- Sets the marriage weight. Set -1 to disallow adoptions relations
		do
			m_adoption_weight := a_weight
		end

	set_natural_parent_weight (a_weight: INTEGER) is
		-- Sets the natural parent/child weight. Set -1 to disallow parent/child relations
		do
			m_natural_parent_weight := a_weight
		end

feature -- Relations

	find_relation : LIST [INTEGER] is
			-- Find the relation. void if no relation is found
			do
				from
					init_nodes (m_start_node.person_id)
				until
					is_search_finished (m_end_node.person_id)
				loop
					move_runners (m_end_node.person_id)
				end

				if m_end_node.has_runner then
					Result := m_end_node.runner.path
				end
			end

feature {NONE} -- Implementation

	init_nodes (first_id: INTEGER) is
			-- Initializes the database for searching a relation
			do
				from
					m_persons.start
				until
					m_persons.after
				loop
					m_persons.item.init_runner (first_id)
					m_persons.forth
				end
			end

	move_runners (last_id : INTEGER) is
			-- Moves the runners to the next node
			do
				from
					m_persons.start
				until
					m_persons.after
				loop
					if m_persons.item.has_runner and m_persons.item.person_id /= last_id then
						m_persons.item.move_runner (m_natural_parent_weight, m_adoption_weight, m_marriage_weight)
					end
					m_persons.forth
				end
			end

	is_search_finished (last_id : INTEGER) : BOOLEAN is
			-- Checks if the search is finished
			local
				active_nodes : LIST[INTEGER]
			do
				create{ARRAYED_LIST[INTEGER]} active_nodes.make (0)
				from
					m_persons.start
				until
					m_persons.after
				loop
					if m_persons.item.has_runner then
						active_nodes.extend (m_persons.item.person_id)
					end
					m_persons.forth
				end

				Result := ((active_nodes.count = 0) or (active_nodes.count = 1 and active_nodes[1] = last_id))
			end


invariant
	invariant_clause: True -- Your invariant here
end
