indexing
	description	: "System's root class"
	date: "$Date: 2008-03-17 16:35:31 +0100 (Mo, 17 Mär 2008) $"
	revision: "$Revision: 5081 $"

class
	ROOT_CLASS


create
	make,
	make_no_read

feature -- Initialization

	make is
			-- Run application.
		do
			initialize_command_table
			initialize_data
			create parser.make
			from
				io.output.put_string ("> ")
				io.output.flush
				io.input.readline
			until
				io.input.end_of_file
			loop
				handle_line (io.input.last_string)
				io.output.put_string ("> ")
				io.output.flush
				io.input.readline
			end
		end

	make_no_read is
			-- Run application.
		do
			initialize_command_table
			initialize_data
			create parser.make
		end

feature -- Command parsing

	handle_line (a_line: STRING) is
			-- Handle the input of line `line'.
		require
			not_void: a_line /= Void
		local
			verb,rest: STRING
			space_pos: INTEGER
		do
			space_pos := a_line.index_of (' ',1)
			if space_pos = 0 then
				verb := a_line
				rest := ""
			else
				verb := a_line.substring (1, space_pos-1)
				rest := a_line.substring (space_pos+1, a_line.count)
			end
			if command_table.has (verb) and command_table.item (verb) /= Void then
				command_table.item (verb).call ([rest])
			elseif not a_line.is_equal ("") then
				print ("Unknown command. Type `help' for a list of available commands.%N")
			end
		end

feature -- Command table

	command_table: HASH_TABLE [PROCEDURE [ANY,TUPLE [STRING]], STRING]

	initialize_command_table is
			-- Initialize the command table.
		do
			create command_table.make (20)
			command_table.extend (agent print_help, "help")
			command_table.extend (agent am_command, "am")
			command_table.extend (agent af_command, "af")
			command_table.extend (agent death_command, "death")
			command_table.extend (agent child_command, "child")
			command_table.extend (agent adopt_command, "adopt")
			command_table.extend (agent marry_command, "marry")
			command_table.extend (agent divorce_command, "divorce")
			command_table.extend (agent show_command, "show")
			command_table.extend (agent cca_command, "cca")
			command_table.extend (agent sr_command, "sr")
			command_table.extend (agent br_command, "br")
			command_table.extend (agent find_command, "find")
			command_table.extend (agent free_command, "free")
			command_table.extend (agent load_command, "load")
			command_table.extend (agent echo_command, "echo")
			command_table.extend (agent trace_command, "trace")
		end

feature -- Help message and errors

	print_help (arg: STRING) is
			-- Print the help text.
		do
			print ("The following commands are available:%N")
			print ("help                    print this help text%N")
			print ("am <id> <fn> <ln> <d>   add a male person%N")
			print ("af <id> <fn> <ln> <d>   add a female person%N")
			print ("death <id> <d>          register the death of <id> on <d>%N")
			print ("child <p> <c>           register a child <c> of <p>%N")
			print ("adopt <p> <c> <d>       register an option of <c> by <p> on <d>%N")
			print ("marry <p1> <p2> <n1> <n2> <d>      marry <p1> to <p2> on <d>%N")
			print ("divorce <p> <n1> <n2> <d>          divorce <p> from partner on <d>%N")
			print ("show <p>                show all information for <p>%N")
			print ("cca <p1> <p1>           print the closest common ancestor of <p1> and <p2>%N")
			print ("sr <p1> <p2>            print the shortest relation between <p1> and <p2>%N")
			print ("br <p1> <p2>            print the best relation between <p1> and <p2>%N")
			print ("find <n>                find all persons with first or last name <n>%N")
			print ("free                    find an unused id%N")
			print ("load <fn>               load commands from file named <fn>%N")
			print ("echo <message>          Print <message> (useful for loading)%N")
			print ("trace on|off            Switch tracing during loading on/off%N")
		end

	print_parse_errors is
			-- Print the error message during parsing.
		local
			errors: LIST [STRING]
		do
			print ("Error parsing arguments:%N")
			from
				errors := parser.error_messages
				errors.start
			until
				errors.off
			loop
				print ("  ")
				print (errors.item)
				print ("%N")
				errors.forth
			end
			print ("Please correct the errors. Type 'help' for a help text.%N")
		end


feature -- Commands

	am_command (arg: STRING) is
			-- Add a male person.
		local
			id: INTEGER
			fn: STRING
			ln: STRING
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			id := parser.last_id
			parser.parse_name
			fn := parser.last_name
			parser.parse_name
			ln := parser.last_name
			parser.parse_date
			d := parser.last_date
			if parser.has_errors then
				print_parse_errors
			else
				if data.has (id) then
					print ("Id ")
					print (id)
					print (" already used.%N")
				elseif not data.valid_date (d) then
					print ("Date ")
					print (d)
					print (" is not valid.%N")
				else
					data.add_male (id, fn, ln, d)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	af_command (arg: STRING) is
			-- Add a female person.
		local
			id: INTEGER
			fn: STRING
			ln: STRING
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			id := parser.last_id
			parser.parse_name
			fn := parser.last_name
			parser.parse_name
			ln := parser.last_name
			parser.parse_date
			d := parser.last_date
			if parser.has_errors then
				print_parse_errors
			else
				if data.has (id) then
					print ("Id ")
					print (id)
					print (" already used.%N")
				elseif not data.valid_date (d) then
					print ("Date ")
					print (d)
					print (" is not valid.%N")
				else
					data.add_female (id, fn, ln, d)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	death_command (arg: STRING) is
			-- Register the death of a person.
		local
			id: INTEGER
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			id := parser.last_id
			parser.parse_date
			d := parser.last_date
			if parser.has_errors then
				print_parse_errors
			else
				if not data.has (id) then
					print ("Id "+id.out+" unknown.%N")
				elseif not data.valid_date (d) then
					print ("Date "+d+" is not a valid date.%N")
				else
					data.register_death (id, d)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	child_command (arg: STRING) is
			-- Register a parent/child relation.
		local
			pid: INTEGER
			cid: INTEGER
		do
			parser.set_input (arg)
			parser.parse_id
			pid := parser.last_id
			parser.parse_id
			cid := parser.last_id
			if parser.has_errors then
				print_parse_errors
			else
				if not data.has (pid) then
					print ("Id "+pid.out+" unknown.%N")
				elseif not data.has (cid) then
					print ("Id "+cid.out+" unknown.%N")
				else
					data.register_child (pid, cid)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	adopt_command (arg: STRING) is
			-- Register the adoption of a child.
		local
			pid: INTEGER
			cid: INTEGER
			n: STRING
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			pid := parser.last_id
			parser.parse_id
			cid := parser.last_id
			parser.parse_name
			n := parser.last_name
			parser.parse_date
			d := parser.last_date
			if parser.has_errors then
				print_parse_errors
			else
				if not data.has (pid) then
					print ("Id "+pid.out+" unknown.%N")
				elseif not data.has (cid) then
					print ("Id "+cid.out+" unknown.%N")
				elseif not data.valid_date (d) then
					print ("Date "+d+" is not valid.%N")
				else
					data.adopt (pid, cid, n, d)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	marry_command (arg: STRING) is
			-- Register a marriage.
		local
			p1: INTEGER
			p2: INTEGER
			n1: STRING
			n2: STRING
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			p1 := parser.last_id
			parser.parse_id
			p2 := parser.last_id
			parser.parse_name
			n1 := parser.last_name
			parser.parse_name
			n2 := parser.last_name
			parser.parse_date
			d := parser.last_date
			if parser.has_errors then
				print_parse_errors
			else
				if not data.has (p1) then
					print ("Id "+p1.out+" unknown.%N")
				elseif not data.has (p2) then
					print ("Id "+p2.out+" unknown.%N")
				else
					data.marry (p1, p2, n1, n2, d)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	divorce_command (arg: STRING) is
			-- Register a divorce.
		local
			p: INTEGER
			n1: STRING
			n2: STRING
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			p := parser.last_id
			parser.parse_name
			n1 := parser.last_name
			parser.parse_name
			n2 := parser.last_name
			parser.parse_date
			d := parser.last_date
			if parser.has_errors then
				print_parse_errors
			else
				if not data.has (p) then
					print ("Id "+p.out+" unknown.%N")
				else
					data.divorce (p, n1, n2, d)
					if not data.last_operation_successful then
						print ("ERROR: "+data.last_error+"%N")
					end
				end
			end
		end

	show_command (arg: STRING) is
			-- Show a person.
		local
			p: INTEGER
			l: LIST [INTEGER]
			d: STRING
		do
			parser.set_input (arg)
			parser.parse_id
			p := parser.last_id
			if parser.has_errors then
				print_parse_errors
			elseif not data.has (p) then
				print ("Id "+p.out+" unknown.%N")
			else
				print ("---------------------------------------------------------%N")
				print ("Name: ")
				print (data.first_name (p))
				print (" ")
				print (data.last_name (p))
				if not data.birth_date (p).is_equal (data.last_name (p)) then
					print (" born ")
					print (data.birth_name (p))
				end
				print ("%N")
				print ("* ")
				print (data.birth_date (p))
				if data.is_dead (p) then
					print (", + ")
					print (data.death_date (p))
				end
				print ("%N")
				if data.father (p) > 0 then
					print ("Father: ")
					print (person_line (data.father (p)))
					print ("%N")
				end
				if data.mother (p) > 0 then
					print ("Mother: ")
					print (person_line (data.mother (p)))
					print ("%N")
				end
				l := data.parents (p)
				from
					l.start
				until
					l.off
				loop
					if l.item /= data.father (p) and l.item /= data.mother (p) then
						print ("Adopted by: ")
						print (person_line (l.item))
						print ("%N")
					end
					l.forth
				end
				l := data.children (p)
				from
					l.start
				until
					l.off
				loop
					if l.item /= data.father (p) and l.item /= data.mother (p) then
						print ("Child: ")
						print (person_line (l.item))
						print ("%N")
					end
					l.forth
				end
				l := data.partners (p)
				from
					l.start
				until
					l.off
				loop
					print ("Married to: ")
					print (person_line (l.item))
					print (" on ")
					print (data.marriage_dates (p).i_th (l.index))
					d := data.divorce_dates (p).i_th (l.index)
					if d /= Void then
						print (" (divorced ")
						print (d)
						print (")")
					end
					print ("%N")
					l.forth
				end
			end
		end

	cca_command (arg: STRING) is
			-- Show closest common ancestor.
		local
			p1,p2,r: INTEGER
		do
			parser.set_input (arg)
			parser.parse_id
			p1 := parser.last_id
			parser.parse_id
			p2 := parser.last_id
			if parser.has_errors then
				print_parse_errors
			elseif not data.has (p1) then
				print ("Id "+p1.out+" unknown.%N")
			elseif not data.has (p2) then
				print ("Id "+p2.out+" unknown.%N")
			else
				r := data.closest_common_ancestor (p1, p2)
				if r = 0 then
					print ("There is no common ancestor.%N")
				else
					print ("The closest common ancestor is:%N")
					print (person_line (r))
					print ("%N")
				end
			end
		end

	sr_command (arg: STRING) is
			-- Show shortest relation.
		local
			p1,p2: INTEGER
			l: LIST [INTEGER]
		do
			parser.set_input (arg)
			parser.parse_id
			p1 := parser.last_id
			parser.parse_id
			p2 := parser.last_id
			if parser.has_errors then
				print_parse_errors
			elseif not data.has (p1) then
				print ("Id "+p1.out+" unknown.%N")
			elseif not data.has (p2) then
				print ("Id "+p2.out+" unknown.%N")
			else
				l := data.shortest_relation (p1, p2)
				if l = Void then
					print ("There is no common ancestor.%N")
				else
					print ("The shortest relation is:%N")
					from
						l.start
					until
						l.off
					loop
						print (person_line (l.item))
						print ("%N")
						l.forth
					end
				end
			end
		end

	br_command (arg: STRING) is
			-- Show best relation.
		local
			p1,p2: INTEGER
			l: LIST [INTEGER]
		do
			parser.set_input (arg)
			parser.parse_id
			p1 := parser.last_id
			parser.parse_id
			p2 := parser.last_id
			if parser.has_errors then
				print_parse_errors
			elseif not data.has (p1) then
				print ("Id "+p1.out+" unknown.%N")
			elseif not data.has (p2) then
				print ("Id "+p2.out+" unknown.%N")
			else
				l := data.best_relation (p1, p2)
				if l = Void then
					print ("There is no common ancestor.%N")
				else
					print ("The shortest relation is:%N")
					from
						l.start
					until
						l.off
					loop
						print (person_line (l.item))
						print ("%N")
						l.forth
					end
				end
			end
		end

	find_command (arg: STRING) is
			-- Find a person by first or last name.
		local
			n: STRING
			l: LIST [INTEGER]
		do
			parser.set_input (arg)
			parser.parse_name
			n := parser.last_name
			l := data.find (n)
			from
				l.start
			until
				l.off
			loop
				print (person_line (l.item))
				print ("%N")
				l.forth
			end
		end

	free_command (arg: STRING) is
			-- Find a free id.
		local
			n: INTEGER
		do
			from
				n := 1
			until
				not data.has (n)
			loop
				n := n + 1
			end
			print ("First free id: ")
			print (n)
			print ("%N")
		end

	load_command (arg: STRING) is
			-- Load a file.
		local
			f: PLAIN_TEXT_FILE
		do
			create f.make (arg)
			f.open_read
			if not f.is_open_read then
				print ("Could not open file %""+arg+"%".%N")
			else
				from
					f.read_line
				until
					f.end_of_file
				loop
					if trace then
						print (">> ")
						print (f.last_string)
						print ("%N")
					end
					handle_line (f.last_string)
					f.read_line
				end
			end
		end

	echo_command (arg: STRING) is
			-- Echo command prints `arg' to the console
		do
			print (arg)
			print ("%N")
		end

	trace_command (arg: STRING) is
			-- Change tracing.
		do
			if arg.is_equal ("on") then
				trace := True
				print ("Tracing on.%N")
			elseif arg.is_equal ("off") then
				trace := False
				print ("Tracing off.%N")
			else
				print ("Use %"trace on%" or %"trace off%".%N")
			end
		end


feature -- Database

	data: GENEALOGY1

	initialize_data is
			-- Initialize data.
		do
			create data.make_empty
		end

feature -- Parser

	parser: SIMPLE_PARSER

feature -- Output

	person_line (p: INTEGER): STRING is
			-- Single line describing person `p'
		require
			person_exists: data.has (p)
		do
			Result := p.out
			Result.append (": ")
			Result.append (data.first_name (p))
			Result.append (" ")
			Result.append (data.last_name (p))
		end

	trace: BOOLEAN
		-- Tracing enabled

end -- class ROOT_CLASS
