indexing
	description: "Objects that execute features of INTEGER_INTERVAL"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	INTEGER_INTERVAL_TESTER

inherit
	CLASS_TESTER

feature -- Basic operations
	execute_small_suite is
			-- Execute small test suite
		do
			test1 (10)
		end

	execute_medium_suite is
			-- Execute medium test suite
		do
			test1 (30)
			test2 (5)
			test3 (5)
			test4 (5)
			test5 (5)
		end

	execute_large_suite is
			-- Execute large test suite
		do
			test1 (300)
			test2 (50)
			test3 (50)
			test4 (50)
			test5 (50)
		end

feature {NONE} -- Parameters
	index_bound: INTEGER is 100

feature {NONE} -- Tests
	test1 (count: INTEGER) is
			-- Test1 (random bounds)
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				randomize
				create interval.make (min, max)
				randomize
				create other.make (min, max)
				interval.adapt (other)
--				randomize				
--				interval.put (max)
				randomize
				interval.resize (min, max)
				randomize_right
				interval.resize_exactly (min, max)
				execute_all_functions
				randomize
				interval.grow (max - min)
				interval.copy (other)
--				interval.wipe_out
				i := i + 1
			end
		end

	test2 (count: INTEGER) is
			-- Test2 (equal bounds)
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				randomize
				create interval.make (min, min)
				randomize
				create other.make (min, max)
				interval.adapt (other)
				interval.resize_exactly (min, min)
--				randomize				
--				interval.put (max)
				randomize
				interval.resize (min, max)
				interval.resize_exactly (min, min)
				randomize_right
				interval.resize_exactly (min, max)
				interval.resize_exactly (min, min)
				execute_all_functions
				randomize
				interval.grow (max - min)
				interval.resize_exactly (min, min)
				if random.decide (0.5) then
					randomize
					other.resize_exactly (min, min)
					random.forth
				else
					random.forth
				end
				interval.copy (other)
--				interval.wipe_out				
				i := i + 1
			end
		end

	test3 (count: INTEGER) is
			-- Test3 ([0, 0])
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				create interval.make (0, 0)
				randomize
				create other.make (min, max)
				interval.adapt (other)
				interval.resize_exactly (0, 0)
--				randomize				
--				interval.put (max)
				randomize
				interval.resize (min, max)
				interval.resize_exactly (0, 0)
				randomize_right
				interval.resize_exactly (min, max)
				interval.resize_exactly (0, 0)
				execute_all_functions
				randomize
				interval.grow (max - min)
				interval.resize_exactly (0, 0)
				if random.decide (0.5) then
					other.resize_exactly (0, 0)
					random.forth
				else
					random.forth
				end
				interval.copy (other)
--				interval.wipe_out				
				i := i + 1
			end
		end

	test4 (count: INTEGER) is
			-- Change comparison criterion
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				create interval.make (1, 0)
				interval.compare_objects
				create other.make (1, 0)
				other.compare_objects
				randomize_right
				interval.resize_exactly (min, max)
				randomize_right
				other.resize_exactly (min, max)
				interval.adapt (other)
		--		randomize				
		--		interval.put (max)
				randomize
				interval.resize (min, max)
				execute_all_functions
				randomize
				interval.grow (max - min)
				interval.copy (other)
		--		interval.wipe_out
				i := i + 1
			end
		end

	test5 (count: INTEGER) is
			-- Test2 (empty interval)
		local
			i: INTEGER
		do
			from
				i := 1
			until
				i > count
			loop
				create interval.make (1, 0)
				randomize
				create other.make (min, max)
				interval.adapt (other)
				interval.resize_exactly (1, 0)
--				randomize				
--				interval.put (max)
				randomize
				interval.resize (min, max)
				interval.resize_exactly (1, 0)
				randomize_right
				interval.resize_exactly (min, max)
				interval.resize_exactly (1, 0)
				execute_all_functions
				randomize
				interval.grow (max - min)
				interval.resize_exactly (1, 0)
				if random.decide (0.5) then
					other.resize_exactly (1, 0)
					random.forth
				else
					random.forth
				end
				interval.copy (other)
--				interval.wipe_out				
				i := i + 1
			end
		end

feature {NONE} -- Implementaion
	interval, other: INTEGER_INTERVAL

	min, max: INTEGER

	randomize is
			-- Randomize `min' and `max'
		do
			min := random.bounded_item (-index_bound, index_bound)
			random.forth
			max := random.bounded_item (-index_bound, index_bound)
			random.forth
		end

	randomize_right is
			-- Randomize `min' and `max' so that max >= min
		do
			min := random.bounded_item (-index_bound, index_bound)
			random.forth
			max := random.bounded_item (min, index_bound)
			random.forth
		end

	execute_all_functions is
			-- Execute all pure functions
		require
			interval_exists: interval /= Void
			other_exists: other /= Void
		local
			n: INTEGER
			b: BOOLEAN
			a: ARRAY [INTEGER]
			l: LINEAR [INTEGER]
			proc: PROCEDURE [ANY, TUPLE [INTEGER]]
			pred: PREDICATE [ANY, TUPLE [INTEGER]]
		do
			if not interval.is_empty then
				n := random.bounded_item (interval.lower, interval.upper)
				random.forth
				n := interval.item (n)
			end
			n := random.bounded_item (-index_bound, index_bound)
			random.forth
			b := interval.has (n)
			n := interval.occurrences (n)
			n := interval.capacity
			n := interval.count
			b := interval.is_equal (other)
			b := interval.all_cleared
			b := interval.extendible
			b := interval.prunable
			a := interval.as_array
			l := interval.linear_representation
			other := interval.index_set
			other := interval.subinterval (interval.lower, interval.upper)
			randomize
			other := interval.subinterval (min, max)
			n := random.bounded_item (-index_bound, index_bound)
			random.forth
			if random.decide (0.5) then
				proc := agent procedure_1 (n, ?)
				pred := agent predicate_1 (n, ?)
			else
				proc := agent procedure_2 (n, ?)
				pred := agent predicate_2 (n, ?)
			end
			random.forth
			interval.do_all (proc) -- Not a function, but doesn't change anything
			b := interval.for_all (pred)
			b := interval.exists (pred)
			b := interval.exists1 (pred)
			n := interval.hold_count (pred)
		end

end
