indexing
	description: "Objects that execute features from ST_SPLITTER class"
	author: ""
	date: "$Date$"
	revision: "$Revision$"

class
	ST_SPLITTER_TESTER

inherit
	CLASS_TESTER

	KL_SHARED_STRING_EQUALITY_TESTER

feature -- Basic operations
	execute_small_suite is
			-- Execute small test suite
		do
			test1 (10)
		end

	execute_medium_suite is
			-- Execute medium test suite
		do
			test1 (15)
			test2 (15)
			test3 (10)
			test4 (10)
		end

	execute_large_suite is
			-- Execute large test suite
		do
			test1 (150)
			test2 (150)
			test3 (100)
			test4 (100)
		end

feature {NONE} -- Tests
	test1 (count: INTEGER) is
			-- Test 1 (split and join with standard separators)
		local
			i: INTEGER
			str: STRING
			b: BOOLEAN
		do
			from
				i := 1
			until
				i > count
			loop
				create splitter.make
				create splitter.make_with_separators ({ST_SPLITTER}.default_separators)
				str := substring (text)

				change_escape_character
				tokens := splitter.split_greedy (str)
				change_tokens
				b := splitter.has_empty (tokens)
				str := splitter.join_unescaped (tokens)
				splitter.set_escape_character (item (escapes))
				change_tokens
				str := splitter.join_greedy (tokens)

				tokens := splitter.split (str)
				change_tokens
				str := splitter.join (tokens)
				i := i + 1
			end
		end

	test2 (count: INTEGER) is
			-- Test 2 (split and join with custom separators)
		local
			i: INTEGER
			str: STRING
			b: BOOLEAN
		do
			from
				i := 1
			until
				i > count
			loop
				create splitter.make_with_separators (nonempty_substring (separators))
				change_escape_character
				splitter.set_separators (nonempty_substring (separators))
				str := substring (text)

				change_escape_character
				tokens := splitter.split_greedy (str)
				change_tokens
				change_escape_character
				b := splitter.has_empty (tokens)
				change_escape_character
				str := splitter.join_unescaped (tokens)
				change_tokens
				splitter.set_escape_character (item (escapes))
				str := splitter.join_greedy (tokens)

				change_escape_character
				tokens := splitter.split (str)
				change_tokens
				splitter.set_escape_character (item (escapes))
				str := splitter.join (tokens)
				i := i + 1
			end
		end

	test3 (count: INTEGER) is
			-- Test 3 (empty string)
		local
			i: INTEGER
			str: STRING
			b: BOOLEAN
		do
			from
				i := 1
			until
				i > count
			loop
				create splitter.make
				create str.make_empty

				change_escape_character
				tokens := splitter.split_greedy (str)
				change_escape_character
				b := splitter.has_empty (tokens)
				change_escape_character
				str := splitter.join_unescaped (tokens)
				splitter.set_escape_character (item (escapes))
				str := splitter.join_greedy (tokens)

				change_escape_character
				tokens := splitter.split (str)
				splitter.set_escape_character (item (escapes))
				str := splitter.join (tokens)
				i := i + 1
			end
		end

	test4 (count: INTEGER) is
			-- Test 4 (string of separators)
		local
			i: INTEGER
			str: STRING
			b: BOOLEAN
		do
			from
				i := 1
			until
				i > count
			loop
				create splitter.make
				str := nonempty_substring (splitter.default_separators)

				change_escape_character
				tokens := splitter.split_greedy (str)
				change_tokens
				change_escape_character
				b := splitter.has_empty (tokens)
				change_escape_character
				str := splitter.join_unescaped (tokens)
				splitter.set_escape_character (item (escapes))
				change_tokens
				str := splitter.join_greedy (tokens)

				change_escape_character
				tokens := splitter.split (str)
				splitter.set_escape_character (item (escapes))
				str := splitter.join (tokens)
				i := i + 1
			end
		end

feature {NONE} -- Implementation
	splitter: ST_SPLITTER
			-- Splitter

	tokens: DS_LIST [STRING]
			-- Splitted string

	change_escape_character is
			-- Randomly set or reset escape character
		require
			splitter_exists: splitter /= Void
		do
			if random.decide (0.5) then
				random.forth
				splitter.set_escape_character (item (escapes))
			else
				random.forth
				splitter.reset_escape_character
			end
		end

	change_tokens is
			-- Change state of `tokens'
		require
			tokens_exists: tokens /= Void
		local
			i: INTEGER
		do
			if not tokens.is_empty then
				i := random.bounded_item (1, tokens.count)
				tokens.go_i_th (i)
				random.forth
			end
			if random.decide (1/3) then
				tokens.set_equality_tester (string_equality_tester)
			elseif random.decide (1/2) then
				tokens.set_equality_tester (case_insensitive_string_equality_tester)
			else
				tokens.set_equality_tester (Void)
			end
		end

	separators: STRING is " %T%N.,:-;`~!*()_+=|[]{}'<>?/"

	escapes: STRING is "%%\@#$^&"
end
